#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include "iolib.h"
#define PROGNAME "dumblog"
#include "freedt.h"

int stamp_ctime = 0;
char *logfile_name;
io *logfile;
buffer line = BUFFER;

void flush_line() {
	if (blength(&line) > 0) {
		buffer out = BUFFER;
		if (stamp_ctime) {
			time_t t = time(NULL);
			const char *s = ctime(&t);
			bappendm(&out, s, strlen(s) - 1);
			bappendc(&out, ' ');
		}
		bappend(&out, &line);
		bappendc(&out, '\n');
		if (bwritea(&out, logfile) < 0)
			die("unable to write to logfile");
		if (isync(logfile) < 0)
			die("unable to sync logfile");
		bfree(&out);
	}
	bfree(&line);
}

void open_logfile() {
	logfile = iopenwa(logfile_name);
	if (!logfile)
		die("unable to open logfile");
}

void close_logfile() {
	flush_line();
	if (iclose(logfile) < 0)
		die("unable to close logfile");
}

void roll_handler(int dummy) {
	close_logfile();
	open_logfile();
}

void quit_handler(int dummy) {
	close_logfile();
	exit(0);
}

int main(int argc, char **argv) {
	struct sigaction sa;

	istdinit();

	while (1) {
		char c = getopt(argc, argv, "c");
		if (c == -1)
			break;
		switch (c) {
		case 'c':
			stamp_ctime = 1;
			break;
		default:
			die("unknown option");
		}
	}

	if ((argc - optind) != 1)
		die("usage: dumblog [-c] logfile");
	logfile_name = argv[optind];

	sigemptyset(&sa.sa_mask);
	sa.sa_flags = 0;

	sa.sa_handler = roll_handler;
	if (sigaction(SIGHUP, &sa, NULL) < 0)
		die("unable to install sighup handler");
	sa.sa_handler = quit_handler;
	if (sigaction(SIGTERM, &sa, NULL) < 0)
		die("unable to install sigterm handler");

	open_logfile();

	while (1) {
		int rc = breadline(&line, ibin);
		if (rc < 0)
			die("unable to read from stdin");
		if (rc == 0)
			break;

		flush_line();
	}

	return 0;
}

